//  EncryptedFileManager
//
//  Created by Jim Range on 11/05/09.
//  Copyright (c) 2009-2012 Heyalda Corporation. All rights reserved.

//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//  
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//  
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.



@interface EncryptedFileManager : NSObject



// Load unencrypted dictionaries
+(NSDictionary*) loadDictionaryFromDocuments:(NSString*) plistFileName;
+(NSDictionary*) loadDictionaryFromResourceBundle:(NSString*) plistFileName;

// Load encrypted dictionaries
+(NSMutableDictionary*) loadFromDocumentsDirectoryEncryptedDictionaryNamed:(NSString*)fileName;
+(NSMutableDictionary*) loadFromResourceBundleEncryptedDictionaryNamed:(NSString*)fileName;

// Write dictionary to encrypted file.
+(BOOL) encryptAndWriteDictionary:(NSDictionary*)dictionary toDocumentsDirectoryWithFileName:(NSString*)fileName;

// Delete a file from the documents directory.
+(BOOL) deleteFileFromDocumentsDirectory:(NSString*)baseFileName;


// File helpers
+(BOOL) doesFileExistInDocuments:(NSString*)fileName;
+(BOOL) doesFileExistInResourceBundle:(NSString*)fileName;


// SHA-1 Hash
+(NSData*)generateSHA1UsingProductKeyForData:(NSData*)sendData;
+(NSString*)getSHA1HashForFileNamed:(NSString*)fileName;



// Raw data i/o

// Read data from either the documents directory or the resourcd bundle, 
// trying the documents directory first and if failure, then the resource bundle.
+(NSData*) loadDataFromFile:(NSString*)fileName;

// Write raw data to documents directory.
+(BOOL) writeData:(NSData*)data toDocumentsDirectoryWithFileName:(NSString*)fileName;

+(NSData*) loadDataFromDocumentsDirectoryWithFileName:(NSString*)fileName;
+(NSData*) loadDataFromResourceBundleWithFileName:(NSString*)fileName;

@end
