//  EncryptedFileManager
//
//  Created by Jim Range on 11/05/09.
//  Copyright (c) 2009-2012 Heyalda Corporation. All rights reserved.

//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//  
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//  
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.



#import "EncryptedFileManager.h"

#import <CommonCrypto/CommonDigest.h>
#import <CommonCrypto/CommonCryptor.h>

#define encKey     @"ENCRYPTION_KEY_abcdefghijklmnopqrstuvwxyz123456789"
#define productKey @"PRODUCT_KEY_abcdefghijklmnopqrstuvwxyz123456789"




#pragma mark -
#pragma mark NSData Encryption Category

// ---------------------------------------------------------
// NSData Category to add Encryption
// ---------------------------------------------------------

@interface NSData (NSDataAESEncryption)
- (NSData *)AES256EncryptWithKey:(NSString *)key;
- (NSData *)AES256DecryptWithKey:(NSString *)key;
@end



@implementation NSData (NSDataAESEncryption)

- (NSData *)AES256EncryptWithKey:(NSString *)key {
    
    char keyPtr[kCCKeySizeAES256+1]; 
    
    bzero(keyPtr, sizeof(keyPtr)); 
    
    [key getCString:keyPtr maxLength:sizeof(keyPtr) encoding:NSUTF8StringEncoding];
    
    NSInteger dataLength = [self length];
    
    size_t bufferSize = dataLength + kCCBlockSizeAES128;
    void *buffer = malloc(bufferSize);
    
    size_t numBytesEncrypted = 0;
    CCCryptorStatus cryptStatus = CCCrypt(kCCEncrypt, kCCAlgorithmAES128, kCCOptionPKCS7Padding,
                                          keyPtr, kCCKeySizeAES256,
                                          NULL ,
                                          [self bytes], dataLength, 
                                          buffer, bufferSize, 
                                          &numBytesEncrypted);
    if (cryptStatus == kCCSuccess) {
        return [NSData dataWithBytesNoCopy:buffer length:numBytesEncrypted];
    }
    
    free(buffer); 
    
    return nil;
}

- (NSData *)AES256DecryptWithKey:(NSString *)key {
	
    char keyPtr[kCCKeySizeAES256+1]; 
	
    bzero(keyPtr, sizeof(keyPtr)); 
	
	[key getCString:keyPtr maxLength:sizeof(keyPtr) encoding:NSUTF8StringEncoding];
	
	NSInteger dataLength = [self length];
	
	size_t bufferSize = dataLength + kCCBlockSizeAES128;
	void *buffer = malloc(bufferSize);
	
	size_t numBytesDecrypted = 0;
	CCCryptorStatus cryptStatus = CCCrypt(kCCDecrypt, kCCAlgorithmAES128, kCCOptionPKCS7Padding,
										  keyPtr, kCCKeySizeAES256,
										  NULL ,
										  [self bytes], dataLength, 
										  buffer, bufferSize, 
										  &numBytesDecrypted);
	
	if (cryptStatus == kCCSuccess) {
		return [NSData dataWithBytesNoCopy:buffer length:numBytesDecrypted];
	}
	
	free(buffer);
	
    return nil;
}


@end





// ---------------------------------------------------------
// Encrypted File Manager
// ---------------------------------------------------------

@interface EncryptedFileManager (private)

+(NSMutableDictionary*) loadEncryptedDictionaryFileNamed:(NSString*)fileName location:(int)zeroForDocsOneForResources;

+(NSMutableDictionary*) getMutableDictFromEncryptedData:(NSData*)encryptedData;
+(NSData*) dictionaryToData:(NSDictionary*)dict;
+(NSMutableDictionary*) dataToDictionary:(NSData*)data;

+(NSString*) fullFilePathToDocumentsDirectoryFileWithBaseFileName:(NSString*)baseFileName;

@end


@implementation EncryptedFileManager


+(NSDictionary*) loadDictionaryFromDocuments:(NSString*) plistFileName {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:plistFileName];
	return [NSDictionary dictionaryWithContentsOfFile:filePath];
    
}

+(NSDictionary*) loadDictionaryFromResourceBundle:(NSString*) plistFileName {
	NSString *filePath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:plistFileName];
	return [NSDictionary dictionaryWithContentsOfFile:filePath];
	
}



+(NSMutableDictionary*) loadFromDocumentsDirectoryEncryptedDictionaryNamed:(NSString*)fileName {
    return [EncryptedFileManager loadEncryptedDictionaryFileNamed:fileName location:0];
}

+(NSMutableDictionary*) loadFromResourceBundleEncryptedDictionaryNamed:(NSString*)fileName {
    return [EncryptedFileManager loadEncryptedDictionaryFileNamed:fileName location:0];
}


+(NSMutableDictionary*) loadEncryptedDictionaryFileNamed:(NSString*)fileName location:(int)zeroForDocsOneForResources {
	
	
	NSString* path;
	
	if (zeroForDocsOneForResources == 0) {
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
		path =  [paths objectAtIndex:0];
	}else {
		path = [[NSBundle mainBundle] resourcePath];
	}
    
	NSString *encryptedFilePath = [path stringByAppendingPathComponent:fileName];	
	
	NSData *fileData = [NSData dataWithContentsOfFile:encryptedFilePath];
	if (fileData == nil) {
		return nil;
	}
	
	NSData *decryptedData = [fileData AES256DecryptWithKey:encKey];
	if (decryptedData == nil) {
		return nil;
	}
	
	NSMutableDictionary* mutableDict = [NSPropertyListSerialization propertyListFromData:decryptedData mutabilityOption:NSPropertyListMutableContainersAndLeaves format:NULL errorDescription:nil];
	
	return mutableDict;
}




+(BOOL) deleteFileFromDocumentsDirectory:(NSString*)baseFileName {
    
	if ([EncryptedFileManager doesFileExistInDocuments:baseFileName]) {
		
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
		NSString *encFileDirectiry = [paths objectAtIndex:0];
		NSString *filePath = [encFileDirectiry stringByAppendingPathComponent:baseFileName];
		
		BOOL isDeleteSuccess = [[NSFileManager defaultManager] removeItemAtPath:filePath error:nil];
		if (!isDeleteSuccess) {
		}
		
		return isDeleteSuccess;
	}
	
	return NO;
    
}



+(BOOL) encryptAndWriteDictionary:(NSDictionary*)dictionary toDocumentsDirectoryWithFileName:(NSString*)fileName{
    
	NSData* plistData = [NSPropertyListSerialization dataFromPropertyList:dictionary format:NSPropertyListXMLFormat_v1_0 errorDescription:nil];
	
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    
    NSString* dataPath = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileName];
	
	NSData *encryptedData = [plistData AES256EncryptWithKey:encKey];
	if (encryptedData == nil) {
		return NO;
	}
    NSLog(@"dataPath:%@", dataPath);
	return [encryptedData writeToFile:dataPath atomically:YES];
}





#pragma mark -
#pragma mark Dictionary Manipulation

+(NSMutableDictionary*) getMutableDictFromEncryptedData:(NSData*)encryptedData {
	NSData *decryptedData = [encryptedData AES256DecryptWithKey:encKey];
	if (decryptedData == nil) {
		return nil;
	}
	
	NSMutableDictionary* mutableDict = [NSPropertyListSerialization propertyListFromData:decryptedData mutabilityOption:NSPropertyListMutableContainersAndLeaves format:NULL errorDescription:nil];
	
	return mutableDict;
}

+(NSData*) dictionaryToData:(NSDictionary*)dict {
	NSData* plistData = [NSPropertyListSerialization dataFromPropertyList:dict format:NSPropertyListXMLFormat_v1_0 errorDescription:nil];
	return plistData;
}

+(NSMutableDictionary*) dataToDictionary:(NSData*)data {
	NSMutableDictionary* mutableDict = [NSPropertyListSerialization propertyListFromData:data mutabilityOption:NSPropertyListMutableContainersAndLeaves format:NULL errorDescription:nil];
	return mutableDict;
}



#pragma mark -
#pragma mark File path helpers

+(BOOL) doesFileExistInDocuments:(NSString*)fileName {
	NSFileManager *fileManager = [NSFileManager defaultManager];
    
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:fileName];
	
	return [fileManager fileExistsAtPath:filePath];
    
}


+(BOOL) doesFileExistInResourceBundle:(NSString*)fileName {
	NSFileManager *fileManager = [NSFileManager defaultManager];
    
	NSString *filePath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:fileName];
    
	return [fileManager fileExistsAtPath:filePath];
    
	
}

+(NSString*) fullFilePathToDocumentsDirectoryFileWithBaseFileName:(NSString*)baseFileName {
    
	NSFileManager *fileManager = [NSFileManager defaultManager];
	
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:baseFileName];
	
	if( [fileManager fileExistsAtPath:filePath]){
		return filePath;
	}else {
		return nil;
	}
	
}


#pragma mark -
#pragma mark SHA1 HASH

+(NSData*)generateSHA1UsingProductKeyForData:(NSData*)sendData {
	
	/// SHA1 context
	CC_SHA1_CTX	sha1Ctx;
	CC_SHA1_Init( &sha1Ctx);
    
	const char * data = (const char *)[sendData bytes];
	CC_SHA1_Update( &sha1Ctx, data, [sendData length] );
    
	NSString* productKeyString = productKey;
	const char * data2 = [productKeyString UTF8String];
	CC_SHA1_Update( &sha1Ctx, data2, strlen(data2) );
    
	unsigned char  pTempKey[20];
	
	CC_SHA1_Final( pTempKey, &sha1Ctx);
    
	NSData* hashValue = [NSData dataWithBytes:pTempKey length:20];
	
	return hashValue;	
}

+(NSString*)getSHA1HashForFileNamed:(NSString*)fileName {
	/// SHA1 context
	CC_SHA1_CTX	sha1Ctx;
	CC_SHA1_Init( &sha1Ctx);
    
    NSData* fileData = [EncryptedFileManager  loadDataFromFile:fileName];
	const char * data = (const char *)[fileData bytes];
	CC_SHA1_Update( &sha1Ctx, data, [fileData length] );
    
	NSString* productKeyString = encKey;
	const char * data2 = [productKeyString UTF8String];
	CC_SHA1_Update( &sha1Ctx, data2, strlen(data2) );
    
    //	NSString *hashStringValue;
	unsigned char  pTempKey[20];
    
    
	CC_SHA1_Final( pTempKey, &sha1Ctx);
    
    NSString* str = [NSString stringWithFormat:@""];
    for (NSInteger i=0; i<20; i++) {
        str = [NSString stringWithFormat:@"%@%X",str, pTempKey[i]];
    }
	return str;
}




#pragma mark -
#pragma mark Raw Data i/o

// Writes raw NSData to a file. Works good for persisting any form of raw data. For example, an array of structs.
+(BOOL) writeData:(NSData*)data toDocumentsDirectoryWithFileName:(NSString*)fileName {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:fileName];
	
	NSFileManager *fileManager = [NSFileManager defaultManager];
    NSError *error;
	
	// First check if the file already exists. If yes, then delete it.
	BOOL fileExists = [fileManager fileExistsAtPath:filePath];
	if (fileExists == YES) {
		BOOL deleteSuccess = [fileManager removeItemAtPath:filePath error:&error];
		if (!deleteSuccess) {
            return NO;
		}
	}
	
	return [data writeToFile:filePath atomically:YES];
	
}


+(NSData*) loadDataFromFile:(NSString*)fileName {
    
	if ([EncryptedFileManager doesFileExistInDocuments:fileName]) {
		return [EncryptedFileManager loadDataFromDocumentsDirectoryWithFileName:fileName];
	}else if ([EncryptedFileManager doesFileExistInResourceBundle:fileName]) {
		return [EncryptedFileManager loadDataFromResourceBundleWithFileName:fileName];
	}else {
		return nil;
	}
	
    
}


+(NSData*) loadDataFromDocumentsDirectoryWithFileName:(NSString*)fileName {
    
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:fileName];
	return [NSData dataWithContentsOfFile:filePath];
	
}

+(NSData*) loadDataFromResourceBundleWithFileName:(NSString*)fileName {
	
	NSString *filePath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:fileName];
	return [NSData dataWithContentsOfFile:filePath];
	
}

@end




